import { AggregateID } from '@mobicoop/ddd-library';
import { AUTHENTICATION_REPOSITORY } from '@modules/authentication/authentication.di-tokens';
import { UpdatePasswordService } from '@modules/authentication/core/application/commands/update-password/update-password.service';
import { Type } from '@modules/authentication/core/domain/username.types';
import { UpdatePasswordRequestDto } from '@modules/authentication/interface/grpc-controllers/dtos/update-password.request.dto';
import { Test, TestingModule } from '@nestjs/testing';
import { UpdatePasswordCommand } from '@modules/authentication/core/application/commands/update-password/update-password.command';

const updatePasswordRequest: UpdatePasswordRequestDto = {
  userId: '165192d4-398a-4469-a16b-98c02cc6f531',
  password: '@Br@ndN3wP@$$w0rd',
};

const mockAuthenticationRepository = {
  findOneById: jest.fn().mockImplementation(() => ({
    id: '165192d4-398a-4469-a16b-98c02cc6f531',
    updatePassword: jest.fn(),
    getProps: jest.fn().mockImplementation(() => ({
      userId: '165192d4-398a-4469-a16b-98c02cc6f531',
      name: 'john.doe@email.com',
      type: Type.EMAIL,
    })),
  })),
  update: jest.fn().mockImplementation(),
};

describe('Update Password Service', () => {
  let updatePasswordService: UpdatePasswordService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: AUTHENTICATION_REPOSITORY,
          useValue: mockAuthenticationRepository,
        },
        UpdatePasswordService,
      ],
    }).compile();

    updatePasswordService = module.get<UpdatePasswordService>(
      UpdatePasswordService,
    );
  });

  it('should be defined', () => {
    expect(updatePasswordService).toBeDefined();
  });

  describe('execution', () => {
    const updatePasswordCommand = new UpdatePasswordCommand({
      userId: updatePasswordRequest.userId,
      password: updatePasswordRequest.password,
    });
    it('should update the password', async () => {
      const result: AggregateID = await updatePasswordService.execute(
        updatePasswordCommand,
      );
      expect(result).toBe('165192d4-398a-4469-a16b-98c02cc6f531');
    });
  });
});
